
from voidpp_tools.json_config import JSONConfigLoader, ConfigLoaderException

loader = JSONConfigLoader(__file__)

def load_config(create = None):
    try:
        config = loader.load('asd-system-config.json', create = create)
    except ConfigLoaderException:
        config = {}
    return config

def save_config(config):
    loader.save(config)



import re
from setuptools.command.install import install as install_base
from asd.tool import ToolHelper

# Note that I use the class name 'install' for the derived class because that is what python setup.py --help-commands will use.
class install(install_base):
    def run(self):
        install_base.run(self)

        # search for the package name for auto register
        installed_files = self.get_outputs()
        pkg_name = None
        # yes, this is a little bit hacky, but I did not found any other way...
        for file in installed_files:
            matches = re.search('dist\-packages\/(.*)\/__init__\.pyc', file)
            if matches:
                pkg_name = matches.group(1)
                break

        if pkg_name is None:
            self.warn('Package name not found. Auto register is not possible.')
            return

        ToolHelper.register_external_tools_module(pkg_name)



import sys
import json
import argparse
import logging

from .tool import ToolBase

logger = logging.getLogger(__name__)

class PathSearchException(Exception):
    pass

class InvalidListIndexException(PathSearchException):
    pass

def is_cont(val):
    return type(val) is dict or type(val) is list

class JSONBrowser(ToolBase):
    name = 'json'
    desc = 'Browse json encoded data by keys.'

    def register(self, parser):
        parser.add_argument('infile', nargs = '?', type = argparse.FileType('r'), default = sys.stdin)
        path_action = parser.add_argument('-p', '--path', help = 'path to sub values', default = '')
        parser.add_argument('-k', '--key_list', action = 'store_true')

        path_action.completer = self.get_path_options
        path_action.quoter = self.format_path_options

    def format_path_options(self, completions):
        logger.debug("Formatting completions: %s", completions)
        try:
            if len(completions) == 1:
                if completions[0][-2:-1] == '.':
                    completions[0] = completions[0][:-1]
                return completions
            else:
                comps = []
                for c in completions:
                    cl = c.split('.')
                    comps.append(cl[-1:][0] if len(cl[-1:][0]) else cl[-2:-1][0]+'.')
                return comps
        except Exception as e:
            logger.exception(e)
            return completions

    def get_path_options(self, prefix, parsed_args, action):
        logger.debug("Get keys for prefix '%s'" % prefix)
        try:
            data = json.load(parsed_args.infile)
            path = prefix.split('.')
            (sub_data, keys) = self.get_tree_keys(data, path)
            logger.debug("Found keys: %s" % keys)
            comps = []
            for k in keys:
                comps.append('.'.join(path[:-1] + ['%s.'%k if is_cont(sub_data[k]) else k]))
            logger.debug("Calced completions: %s" % comps)
            return comps
        except Exception as e:
            logger.exception(e)
            return []

    def get_tree_keys(self, data, path):
        if type(data) is dict:
            keys = data.keys()
            if len(path) and path[0] in keys:
                return self.get_tree_keys(data[path[0]], path[1:])
            else:
                return data, keys
        elif type(data) is list:
            idx = int(path[0]) if len(path) and len(path[0]) else -1
            if idx >= 0 and idx < len(data):
                return self.get_tree_keys(data[idx], path[1:])
            else:
                return data, range(len(data))
        return []

    def get_sub_value(self, data, path_array):
        if len(path_array) == 0 or path_array[0] == '':
            return data
        key = path_array.pop(0)
        data_type = type(data)
        if data_type is list:
            if key.isdigit():
                key = int(key)
                if key < 0 or key >= len(data):
                    raise PathSearchException()
            elif key.find(':') != -1:
                sub_keys = key.split(':')
                start_key = sub_keys[0] if len(sub_keys[0]) else 0
                end_key = sub_keys[1] if len(sub_keys[1]) else len(data)
                try:
                    start_key = int(start_key)
                    end_key = int(end_key)
                except ValueError as e:
                    raise InvalidListIndexException()
                return data[start_key:end_key]
            else:
                raise PathSearchException()
        else:
            if key not in data:
                raise PathSearchException()
        return self.get_sub_value(data[key], path_array)

    def process(self, args):
        try:
            data = json.load(args.infile)
            sub_value = self.get_sub_value(data, args.path.split('.'))
            if is_cont(sub_value):
                if args.key_list:
                    if type(sub_value) is dict:
                        print json.dumps(sub_value.keys(), indent = 4)
                    else:
                        print range(len(sub_value))
                else:
                    print json.dumps(sub_value, indent = 4)
            else:
                print sub_value

        except ValueError as e:
            print 'Could not decode json: %s' % e
        except InvalidListIndexException as e:
            print 'Invalid list index: "%s"' % args.path
        except PathSearchException as e:
            print 'Could not find the specified path: "%s"' % args.path






import sys
import inspect
import importlib
from abc import ABCMeta, abstractmethod
from .system_config import load_config, save_config

class ToolBase(object):
    __metaclass__ = ABCMeta

    name = None
    desc = None
    example_config = {}

    def __init__(self, config = {}):
        self.config = config

    @abstractmethod
    def register(self, parser):
        pass

    @abstractmethod
    def process(self, args):
        pass

class ToolHelper():

    @staticmethod
    def _get_tool_classes(name):
        external_module = importlib.import_module(name)
        for type_name, type in inspect.getmembers(external_module):
            if inspect.isclass(type) and type is not ToolBase and ToolBase in inspect.getmro(type):
                yield type

    @staticmethod
    def register_external_tools_module(name):
        config = load_config('/etc')
        if 'external_tools' not in config:
            config['external_tools'] = dict()
        config['external_tools'][name] = {}
        for type in ToolHelper._get_tool_classes(name):
            if type.name not in config['external_tools'][name]:
                config['external_tools'][name][type.name] = dict(
                    enabled = True,
                    config = type.example_config,
                )
        save_config(config)

    @staticmethod
    def load_external_tools(system_config, user_config):
        tools = []
        for name in system_config:
            for type in ToolHelper._get_tool_classes(name):
                cfg = system_config[name][type.name]
                if name in user_config and type.name in user_config[name]:
                    cfg = user_config[name][type.name]
                if cfg['enabled']:
                    tools.append(type(cfg['config']))
        return tools



import logging
from argcomplete import CompletionFinder
from argcomplete.my_argparse import action_is_satisfied, action_is_open

logger = logging.getLogger(__name__)

class CustomCompletionFinder(CompletionFinder):

    def quote_completions(self, completions, cword_prequote, first_colon_pos):
        comps = super(CustomCompletionFinder, self).quote_completions(completions, cword_prequote, first_colon_pos)

        for active_action in self.active_parsers[-1].active_actions:
            if not active_action.option_strings:  # action is a positional
                if action_is_satisfied(active_action) and not action_is_open(active_action):
                    continue

            quoter = getattr(active_action, "quoter", None)

            if quoter is not None:
                logger.debug("Found custom quoter callback: %s" % quoter)
                comps = quoter(comps)

        return comps

custom_auto_complete = CustomCompletionFinder()



import os
from voidpp_tools.json_config import JSONConfigLoader, ConfigLoaderException

loader = JSONConfigLoader(__file__)

def load_config():
    return loader.load('asd-config.json', create = os.path.expanduser('~'))

def save_config(config):
    loader.save(config)



from setuptools import setup, find_packages

setup(
    name = "asd",
    description = "Various command line helper scripts`",
    version = "1.1.1",
    author = 'Lajos Santa',
    author_email = 'santa.lajos@coldline.hu',
    url = 'https://github.com/voidpp/asd.git',
    install_requires = [
        "argcomplete==1.0.0",
        "voidpp-tools>=1.1.1",
    ],
    scripts = [
        'bin/asd',
    ],
    include_package_data = True,
    packages = find_packages(),
)
